#!/usr/bin/perl
# Parsim v1.0.7g beta
# Modified 11 November, 2002
# (c) Jonathan Jeffery 2002
#
# Reads PAUP log files of autoapomorphy lists and cracks the data
# using the ASCAS parsimony cracking method

#set-up stuff
use utf8;
use File::Basename;
$version = "Parsimov 1.0.7g beta";

@SuffixList = ('\..*');
#cheat to get local dir syntax
@LocalPathDetails = fileparse('DoesNotExistAtAll.Hmmm', @SuffixList);
$LocalPath = $LocalPathDetails[1];
#open (BUM, '>', 'Who.txt'); #spare files for bug-hunting
#open(ARSE, '>', "What.txt");

print "\n\n   ?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?\n\n";
print     "       ASCAS Parsimony-Based Cracking Method\n\n";
print     "   ?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?=?\n\n";

print     "     Jonathan Jeffery  &  Olaf Bininda-Emonds\n";
print     "       Univ. Leiden            Germany\n";


require 5.6.0;

@TestManFile = ('f', 'm');

while(1){
	print "\nDo you want to use a batch-file or enter the settings manually? \(f\\m\)\n   ";
	$Man_or_file = <STDIN>;
	chomp($Man_or_file);
	$Man_or_file =~ tr/A-Z/a-z/;
	$GreppingTest = grep(/$Man_or_file/, @TestManFile);
	if($GreppingTest == 0){next}
	else{last}
	}
if($Man_or_file eq "m"){&Manual_Input}
else{&Batch_File}



#################################################
#sub to open batch file for input details
sub Batch_File {

	print "\nWhat batch-file do you want to use?\n   ";
	#print "File must be in the local directory:\n";

	while(1){
		$InputBatchFile = <stdin>;
		chomp ($InputBatchFile);
		if((-e $InputBatchFile) == 1){last};
		print "\nCan\'t find that file (wrong spelling or file path?).";
		print "\nPlease try again:\n   ";
		}

	@BatchFileDetails = fileparse($InputBatchFile, @SuffixList);
	$MyPath = $BatchFileDetails[1];
	$InFile = $BatchFileDetails[0] . $BatchFileDetails[2];

	$InputBatchFile =~ tr/A-Z/a-z/;
	open(BATCH,  '<', $InputBatchFile);


	#Put whole infile into a list
	@InputBatchList = <BATCH>;

	$Mac = "\x{0D}";#chr(015);
	$PC = "\x{0A}";#chr(012);
	for ($i = 0; $i <= $#InputBatchList; $i++) {
		$line = $InputBatchList[$i];
		$line =~ s/$Mac|$PC/\n/g;
		$line =~ tr/\n/\n/s;

		@TempList = split(/\n/, $line);
		for ($TT = 0; $TT <= $#TempList; $TT++){
			$TempLine = $TempList[$TT];
			unless(length($TempLine) == 0){
				push(@FinalBatchList, $TempLine)
				}
			}
		}

	foreach $BatchLine (@FinalBatchList){

		$Igloo = int(rand(1000));
		@Details_of_Run = split(/\t/, $BatchLine);

		#input file
		$OriginalInFile = $Details_of_Run[0];
		$OriginalInFile =~ tr/A-Z/a-z/;
		@PathOriginalInFile = fileparse($OriginalInFile, @SuffixList);

		if($PathOriginalInFile[1] eq $LocalPath){
			$OriginalInFile = $MyPath . $OriginalInFile;
			@PathOriginalInFile = fileparse($OriginalInFile, @SuffixList);
			}
		$ScreenOrigInFile = $PathOriginalInFile[0] . $PathOriginalInFile[2];

		open(NEXUS,  '<', $OriginalInFile) or die "The file $OriginalInFile could not be found.\n";
		$FileToClean = "NEXUS";
		@MasterFileList = ();
		@MasterNodeList = ();
		@MasterNodeStore = ();
		@TreeShape = ();
		@NexusList = ();

		#outputfile
		$Extension = ".crk";
		$OutFile = $Details_of_Run[1];
		@PathOutFile = fileparse($OutFile, @SuffixList);
		if($PathOutFile[1] eq $LocalPath){
			$PathOutFile[1] = $PathOriginalInFile[1];
			}

		$OutFile = $PathOutFile[0];
		&Out_length_check;

		$OutFile = $PathOutFile[1] . $OutFile . $Extension;
		open(OUTPUT, '>' , $OutFile) or die "The file $OutFile could not be opened.\n";;

		$Unambig = $Details_of_Run[2];
		$Unambig =~ tr/A-Z/a-z/;
		$Exhaustive = $Details_of_Run[3];
		$Exhaustive =~ tr/A-Z/a-z/;
		$DeleteOldFile = $Details_of_Run[4];
		chomp($DeleteOldFile);
		$DeleteOldFile =~ tr/A-Z/a-z/;

		$PrintAnswer = "All";
		$Printanswer2 = "used if possible";

		if($Unambig eq "u"){$PrintAnswer = "Only unambiguous"}
		if($Exhaustive eq "n"){$Printanswer2 = "not used"}


		print OUTPUT "Summary of ASCAS-cracked data from \"$OriginalInFile\"\n";
		&CreateDate;
		print OUTPUT "Created $Days[$CreateDay], $CreateDate $Months[$CreateMonth] $CreateYear at ";
		printf OUTPUT ("%02d:%02d:%02d", $CreateHour, $CreateMin, $CreateSec);
		print STORE " by $version\n";

		if($DeleteOldFile eq 'y'){
			print OUTPUT "\nFor each node, a record of all the shortest";
			print OUTPUT "\nruns has been saved as a .out file.";
			print OUTPUT "\nThese files have all have the suffix \"-%$Igloo\"\n";
			}
		else{
			print OUTPUT "\nFor each node, results of all the cracking runs and a record of all the";
			print OUTPUT "\nshortest runs have been saved as a .run and .out file, respectively";
			print OUTPUT "\nThese files have all have the suffix \"-%$Igloo\"\n";
			}
		&MAIN_PROGRAM;
		}

	}
#end of sub to open batch file for input details
#################################################



####################################################
#Sub to get the input manually
sub Manual_Input {
	$Igloo = int(rand(1000));

	##################################################
	#Startup

	# Get input file name and open it
	print "\nWhat file do you want to use?\n";
	print "File must be in the local directory, and be in Nexus format:\n   ";

	while(1){
		$OriginalInFile = <stdin>;
		chomp ($OriginalInFile);
		if((-e $OriginalInFile) == 1){last};
		print "\nCan\'t find that file (wrong spelling or file path?).";
		print "\nPlease try again:\n   ";
		}

	$OriginalInFile =~ tr/A-Z/a-z/;
	@PathOriginalInFile = fileparse($OriginalInFile, @SuffixList);
	$ScreenOrigInFile = $PathOriginalInFile[0] . $PathOriginalInFile[2];

	open(NEXUS,  '<', $OriginalInFile);
	$FileToClean = "NEXUS";
	@MasterFileList = ();


	@TestList = ('a', 'u');
	while(1){
		print "\nDo you want to use all EPs or\njust unambiguously located ones? \(a\\u\)   ";
		$Answer = <STDIN>;
		chomp($Answer);
		$Answer =~ tr/A-Z/a-z/;
		$TestInput = grep(/$Answer/, @TestList);
		if($TestInput == 0){next}
		else{$Unambig = $Answer; last}
		}

	@TestList = ('y', 'n');
	while(1){
		print "\nDo you want to use the exhaustive search\nif possible? \(8 events or less; y\\n\)   ";
		$Answer = <STDIN>;
		chomp($Answer);
		$Answer =~ tr/A-Z/a-z/;
		$TestInput = grep(/$Answer/, @TestList);
		if($TestInput == 0){next}
		else{$Exhaustive = $Answer; last}
		}


	@TestList = ('y', 'n');
	while(1){
		print "\nDo you want to delete the working-files\nafter use? (recommended for big data-sets; y\\n\)   ";
		$Answer = <STDIN>;
		chomp($Answer);
		$Answer =~ tr/A-Z/a-z/;
		$TestInput = grep(/$Answer/, @TestList);
		if($TestInput == 0){next}
		else{$DeleteOldFile = $Answer; last}
		}


	$PrintAnswer = "All";
	$Printanswer2 = "used if possible";

	if($Unambig eq "u"){$PrintAnswer = "Only unambiguous"}
	if($Exhaustive eq "n"){$Printanswer2 = "not used"}

	###############################################
	#Open output file
	$FileTypeToOpen = "an output";
	$Extension = ".crk";
	&OpenOut;
	open(OUTPUT, $OutType, $OutFile);

	print OUTPUT "Summary of ASCAS-cracked data from \"$OriginalInFile\"\n";
	&CreateDate;
	print OUTPUT "Created $Days[$CreateDay], $CreateDate $Months[$CreateMonth] $CreateYear at ";
	printf OUTPUT ("%02d:%02d:%02d", $CreateHour, $CreateMin, $CreateSec);
	print OUTPUT " by $version\n";

	if($DeleteOldFile eq 'y'){
		print OUTPUT "\nFor each node, a record of all the shortest";
		print OUTPUT "\nruns has been saved as a .out file.";
		print OUTPUT "\nThese files have all have the suffix \"-%$Igloo\"\n";
		}
	else{
		print OUTPUT "\nFor each node, results of all the cracking runs and a record of all the";
		print OUTPUT "\nshortest runs have been saved as a .run and .out file, respectively";
		print OUTPUT "\nThese files have all have the suffix \"-%$Igloo\"\n";
		}
	&MAIN_PROGRAM;
	}
########################################
#End of Manual_Input sub




########################################################
########################################################
#sub containing the whole program!
sub MAIN_PROGRAM {

@MasterCrackArray2 = ();
@OriginalNEXUSstore = ();

print OUTPUT "\n$PrintAnswer characters used.  Exhaustive searches $Printanswer2.";

@StartTime = localtime(time);

@InputNexus = ();
#Put whole infile into a list
@InputNexus = <NEXUS>;

#Goto 'import' sub to clean up file, correct for Mac/PC and make a 3D Array of the data
@FileNexus = ();
@FileNexus = @InputNexus;
&ImportFile;

close(OUTPUT);



##########################################
#Go to each node on the list, load in the data and convert it
#to crackable format
$Looking = 0;
@MasterNodeList =();
$MasterNodeCounter = 0;
$HighestChar = 0;

for($X = 0; $X <= $#OriginalNEXUSstore; $X++){
	@LineLengthTest = @{$OriginalNEXUSstore[$X]};
	if($#LineLengthTest > 6){
		&select_node;
		&FindHighest;
		push(@MasterNodeStore, [@NodeInQuestion]);
		}
	}

&createlookup;
&ConvertChars;


##########################################
#loop to get big array of cracking assoc arrays for each node.
for($ZZTop = 0; $ZZTop <= $#MasterNodeStore; $ZZTop++){
	&ForCrackingList;
	push(@MasterCrackArray2, [%CrackingList]);
	%InputArray = %CrackingList; #@{$MasterCrackArray2[$ZZTop]};
	}


##########################################
#Initialise a run for a node
sub Initialise_Run {
	%TempScore = ();
	%Twins = ();
	%TempScore = (
		TotalScore => "0",
		Conflict => [()],
		Twins => [%Twins]
		);
	$TestEvent = 0;
	}

##########################################
#end of Initialise a run for this node


@TempLookList = ();
@MasterLookList = ();
@NotWorthIt = ();

for($ZZTop = 0; $ZZTop <= $#MasterNodeList; $ZZTop++){
	$Wherethefuck = 0;
	#Open data file for this node...
	@FileToPrintonScreen =split(/-I-/, $MasterNodeList[$ZZTop]);
	@FileToPrintonScreen2 = ();
	foreach $NodeChange (@FileToPrintonScreen){
		$NodeChangeTemp = $NodeChange;
		if($NodeChangeTemp =~ /node_/){
			$NodeChangeTemp  =~ s/node_/Node /g;
			}
		$NodeChangeTemp = "\u$NodeChangeTemp";
		push(@FileToPrintonScreen2, $NodeChangeTemp);
		}
	@FileToPrintonScreen = @FileToPrintonScreen2;

	print "\nNow doing $FileToPrintonScreen[0] to $FileToPrintonScreen[1], source data from $ScreenOrigInFile";

	$WorthIt = 0;
	&ForCrackingList;

	if($WorthIt == 0){
		open(OUTPUT, '>>', $OutFile);
		print OUTPUT "\n\n* $FileToPrintonScreen[0] --> $FileToPrintonScreen[1] does not have any apomorphies under current criteria";
		close(OUTPUT);

		$PushOver = " $ZZTop ";
		push(@NotWorthIt, $PushOver);
		next;
		}

	@FileToSave =  split(/-I-/, $MasterNodeList[$ZZTop]);
	$FileFromIn = $FileToSave[0];
	$FileToIn = $FileToSave[1];
	until(length($FileFromIn) <= 8){
		chop($FileFromIn);
		}
	until(length($FileToIn) <= 7){
		chop($FileToIn);
		}

	$FileToSave =~ s/-I-/---/g;
	$FileToSave =  $PathOutFile[1] . "$ZZTop." . $FileFromIn . "---" . $FileToIn . "-%$Igloo.run";
	open(DATA, '>', $FileToSave);
	push(@MasterFileList, $FileToSave);

	#Do a threshold search...
	$Wherethefuck = 0;
	&Initialise_Run;
	$WhichExhaust = -1;
	$abort = 0;
	$HSearch = 1;
	&ForCrackingList;
	%InputArrayThresh = %CrackingList;
	&ThresholdSearch;
	#print ARSE "\nafter \&ThresholdSearch the \$abortThreshold is $abortThreshold";

	#Do a free seach (on Find_best)...
	$Wherethefuck = 1;
	for ($BFUB = 1;  $BFUB <= 100; $BFUB++){
		&Initialise_Run;
		$abort = 0;
		&ForCrackingList;
		%NodeInputArray = %CrackingList;
		%PrintArrayTemp = %NodeInputArray;
		print DATA "Free run $BFUB\n";
		#print ARSE "\n\nFree run $BFUB";
		@This_Doing_List = ();
		$AdjustedGuide = 2;
		&Do_Node;
		#print ARSE "\nafter Free run $BFUB the \$abortThreshold is $abortThreshold, but \$TempScore\{\"TotalScore\"\} is " . $TempScore{"TotalScore"};

		if(($TempScore{"TotalScore"} < $abortThreshold) and ($abort == 0)){
			@temptest = @{$TempScore{"Conflict"}};
			if(@temptest > 0){
				print DATA "\n-------------- Shorter, but with conflicts ------------";
				if(@This_Doing_List > 0 ){print DATA "\nFinal order used is @This_Doing_List\n";}
				else{print DATA "\nFinal order null (no nontrivial seeds)\n"}
				&Print_Final;
				}
			else{
				$abortThreshold = $TempScore{"TotalScore"};
				close(DATA);
				open(DATA, '>', $FileToSave);
				#print ARSE "\nAdjusted the \$abortThreshold to $abortThreshold, as \$TempScore\{\"TotalScore\"\} was " . $TempScore{"TotalScore"};
				print DATA "New Threshold!\n";
				print DATA "Free run $BFUB\n";
				$BFUB = 1;
				}
			}
		}
	print DATA "End of free runs\n\n\n";

	#Now find search sets...
	&Initialise_Run;
	&ForCrackingList;
	%InputArrayExhaust = %CrackingList;
	@TotalHolder = ();
	&FindExhastSets;


	@TempLookList = ($HowLooked, $HowManyIterations, $Nontrivial);
	push(@MasterLookList, [@TempLookList]);

	$Wherethefuck = 2;
	#Finally do a guided search...
	$HowFar = $#TotalHolder;
	@This_Doing_List = ();

	for ($BFUB = 0;  $BFUB <= $HowFar; $BFUB++){
		&Initialise_Run;
		$HSearch = 2;
		@TempGuide = @{$TotalHolder[$BFUB]};
		#print ARSE "\nUsing @TempGuide";
		@TempGuide2 = ();
		foreach $DirTemp (@TempGuide){
			$TempVar++;
			@CheckPolarity = split(/_/, $DirTemp);
			unless($CheckPolarity[1] eq "N"){
				%TempCharScore = ();
				$PreSetPol = $CheckPolarity[1];

				$TempScore{"$CheckPolarity[0]"} = [$PreSetPol, [%TempCharScore], "P"];
				}
			push(@TempGuide2, $CheckPolarity[0])
			}
		@TempGuide = @TempGuide2;

		$abort = 0;
		$WhichExhaust = -1;
		if($Nontrivial > 0){
			print DATA "Guided run " . ($BFUB + 1) . ", using @TempGuide";
			#print ARSE "\nGuided run " . ($BFUB + 1) . ", using @TempGuide";
			}
		else{print DATA "Guided run " . ($BFUB + 1) . ", no nontrivial seeds";}


		$AdjustedGuide = 0;
		&ForCrackingList;
		%NodeInputArray = %CrackingList;
		%PrintArrayTemp = %NodeInputArray;
		&Do_Node;
		if($abort == 1){print DATA "\n"}
		elsif($TempScore{"TotalScore"} < $abortThreshold){
			@temptest = @{$TempScore{"Conflict"}};
			if(@temptest > 0){
				print DATA "\n-------------- Shorter, but with conflicts ------------";
				print DATA "\nFinal order used is @This_Doing_List\n";
				#print ARSE "\n\$abortThreshold remains $abortThreshold, as conflicts generated \$TempScore\{\"TotalScore\"\} was " . $TempScore{"TotalScore"};
				&Print_Final;
				}
			else{
				$abortThreshold = $TempScore{"TotalScore"};
				close(DATA);
				open(DATA, '>', $FileToSave);
				print DATA "New Threshold!\n";
				print DATA "Guided run " . ($BFUB + 1) . ", using @TempGuide\n";
				print DATA "\nFinal order used is @This_Doing_List\n";
				#print ARSE "\nAdjusted the \$abortThreshold to $abortThreshold, as \$TempScore\{\"TotalScore\"\} was " . $TempScore{"TotalScore"};
				&Print_Final;
				}
			}
		}

	print DATA "End of guided runs\n\n";
	&Initialise_Run;
	&ForCrackingList;
	%PrintArrayTemp = %CrackingList;
	&tempPrint2;
	close(DATA);
	}

#end of looking at nodes

#remove nodes with no synaps from MasterNodeList
#

@NotRemoved = ();
for($Removal = 0; $Removal <= $#MasterNodeList; $Removal++){
	$IsItAllWorthIt = grep(/ $Removal /, @NotWorthIt);
	if($IsItAllWorthIt >= 1){
		next;
		}
	else{
		push(@NotRemoved, $MasterNodeList[$Removal]);
		}
	}
@MasterNodeList = @NotRemoved;

#end of removing nodes


#Now read in the data, remove duplicates and print a consensus file...
for($RunRunRun = 0; $RunRunRun <= $#MasterFileList; $RunRunRun++){
	%ConflictCons = ();
	%ConflictUnique = ();
	%ConflictingRuns = ();
	%CompareListFinal = ();
	%Consensus = ();

	$ReadInFile = $MasterFileList[$RunRunRun];
	$ReadInNode = $MasterNodeList[$RunRunRun];
	@TempLookList = @{$MasterLookList[$RunRunRun]};
	#print BUM "\nopening $ReadInFile, and \@TempLookList is @TempLookList";

	$OutStore = $ReadInFile;
	$OutStore  =~ s/\.run$/\.out/g;
	open(STORE, '>', "$OutStore");

	@FinalFromTo = split(/-I-/, $ReadInNode);
	@FinalFromTo2 =();
	foreach $NodeChange (@FinalFromTo){
		$NodeChangeTemp = $NodeChange;
		if($NodeChangeTemp =~ /node_/){
			$NodeChangeTemp  =~ s/node_/Node /g;
			}
		$NodeChangeTemp = "\u$NodeChangeTemp";
		push(@FinalFromTo2, $NodeChangeTemp);
		}
	@FinalFromTo = @FinalFromTo2;

	print "\nNow reading-in $FinalFromTo[0] to $FinalFromTo[1], source data from $ScreenOrigInFile";

	print STORE "ASCAS-cracked data for $FinalFromTo[0] --> $FinalFromTo[1]; Source data from \"$OriginalInFile\"\n";
	&CreateDate;
	print STORE "Created $Days[$CreateDay], $CreateDate $Months[$CreateMonth] $CreateYear at ";
	printf STORE ("%02d:%02d:%02d", $CreateHour, $CreateMin, $CreateSec);
	print STORE " by $version\n";

	if($Unambig eq "a"){print STORE "$Optimise character-state optimization used\n";}
	print STORE "$TempLookList[2] characters with informative movement\n$TempLookList[0] ($TempLookList[1] iterations)\n";

	&ReadInCrackData;

	open(OUTPUT, '>>', $OutFile);
	&PrintOutAll;
	close(OUTPUT);
	close(STORE);
	}
close(ARSE);

open(OUTPUT, '>>', $OutFile);
&EndTime;
close(OUTPUT);

###################################################################
###################################################################
###################################################################
# ONLY SUBS BELOW THIS POINT!!!!!!!!!!!!!!
###################################################################
###################################################################


##################################################
#sub temp printout2
#prints current status of %Inputarray and %tempscore

sub tempPrint2 {
	@TempKeys = sort{$a <=> $b}(keys(%PrintArrayTemp));

	print DATA "\n===========================================\n";
	print DATA "Original Input Array\n";
	foreach $TMTP (@TempKeys){
		%temp1 = @{$PrintArrayTemp{"$TMTP"}[2]};
		%temp2 = @{$PrintArrayTemp{"$TMTP"}[3]};
		$printchar = $TMTP;
		$temp3 = $PrintArrayTemp{"$TMTP"}[0];
		$temp4 = $PrintArrayTemp{"$TMTP"}[1];
		print DATA "\nChar is $printchar,\tRelTotal is $temp3,\tAbsTotal is $temp4,\tL ";

		@TempLate = sort{$a <=> $b}(keys(%temp1));
		foreach $YEP (@TempLate){
			print DATA "$YEP,";
			}
		print DATA "  E ";
		@TempEarly = sort{$a <=> $b}(keys(%temp2));
		foreach $YEP (@TempEarly){
			print DATA "$YEP,";
			}
		}
	print DATA "\n\n";
	}
#end of temp printout
##################################################


##############################################################
#sub to build 3D list of every possible order of a set of events
#Events come from list @ExhastSet
sub BuildSet {
	@Build = ();

	if(@ExhastSet <= 4){
		$quickie = $ExhastSet[0] . "_E";
		push(@TotalHolder, [$quickie]);
		$quickie = $ExhastSet[0] . "_L";
		push(@TotalHolder, [$quickie]);
		$HowLooked = "Exhaustive search using all possible polarised seed-combinations";
		}
	else{
		$HowLooked = "Thorough search using all possible unpolarised seed-combinations";
		$quickie = $ExhastSet[0] . "_N";
		@TotalHolder = ([$quickie]);
		}

	for($First = 1; $First <= $#ExhastSet; $First++){
		@TempTotalHolder = ();
		for($AddChar = 0; $AddChar <= $#TotalHolder; $AddChar++){
			@Build = @{$TotalHolder[$AddChar]};
			for($Place = 0; $Place <= $#Build + 1; $Place++){
				if(@ExhastSet >= 5){
					$TheChar = $ExhastSet[$First] . "_N";
					}
				else{
					$TheChar = $ExhastSet[$First] . "_E";
					}
				@temp2 = @Build;
				splice(@temp2, $Place, 0, $TheChar);
				push(@TempTotalHolder, [@temp2]);

				if(@ExhastSet <= 4){
					$TheChar = $ExhastSet[$First] . "_L";
					@temp2 = @Build;
					splice(@temp2, $Place, 0, $TheChar);
					push(@TempTotalHolder, [@temp2]);
					}
				}
			}
		@TotalHolder = @TempTotalHolder;
		}
	}
#end of sub to build 3D list of every possible order of a set of events
###############################################################


#############################################################################
#sub to build up a list of exhaustive event sequences or a list of every passible pair of events
#takes %InputArray and @CycleLength
#Gives  @TotalHolder
sub FindExhastSets {
	%InputArrayTwZo = %InputArrayExhaust;
	&Wipe_Twins;
	&Wipe_Zeros;
	$HowLooked = "";
	$HowManyIterations = 0;
	$Nontrivial = 0;
	$CompleteSet = 0;
	@tempkeys = sort{$a <=> $b}(keys(%InputArrayTwZo));
	$Nontrivial = @tempkeys;

	if($Nontrivial == 0){
		$HowManyIterations = 0;
		$HowLooked = "No nontrivial seeds";
		}

	elsif(((@tempkeys <= 7) and ($Exhaustive eq "y")) or ((@tempkeys <= 4) and (@tempkeys > 0))){
		@ExhastSet = @tempkeys;
		&BuildSet;
		$CompleteSet = 1;
		$HSearch = 2;
		}
	else{
		@TotalHolder = ();
		@HeurSet = @tempkeys;
		@tempFinal = ();
		$SetTestesr = 0;
		&HeursetBuild;
		$HSearch = 2;
		}
	$HowManyIterations = @TotalHolder;
	}

#end of sub to build up a list of exhaustive event sequences
#############################################################################


##########################################
#sub to make a list of every possible comb of two events
sub HeursetBuild{

 	$HowLooked = "Heuristic search using two unpolarised seeds";
	for($ThisChar = 0; $ThisChar <= $#HeurSet; $ThisChar++){
		for($OtherChar = $ThisChar + 1; $OtherChar <= $#HeurSet; $OtherChar++){
			$FirstChar = $HeurSet[$ThisChar] . "_N";
			$SecondChar = $HeurSet[$OtherChar] . "_N";
			@temppair = ($FirstChar, $SecondChar);
			push(@TotalHolder, [@temppair]);

			if(@tempkeys <= 30){
				&AddAThird;
				$HowLooked = "Heuristic search using three unpolarised seeds";
				}

			@temppair = ($SecondChar, $FirstChar);
			push(@TotalHolder, [@temppair]);

			if(@tempkeys <= 30){
				&AddAThird;
				@TotalHolder = @tempFinal;
				}
			}
		}
	}
#end of sub to make a list of every possible comb of two events
##########################################


####################################################
#sub to add a third char in every position on an existing two char list
sub AddAThird{
	for($NextChar = $OtherChar + 1; $NextChar <= $#HeurSet; $NextChar++){
		$ThirdChar = $HeurSet[$NextChar] . "_N";

		for($Place = 0; $Place <= $#temppair + 1; $Place++){
			@tempSecond = @temppair;
			splice(@tempSecond, $Place, 0, $ThirdChar);
			push(@tempFinal, [@tempSecond]);
			}
		}
	}
#end of sub to add a third char
###############################


####################################
#sub to Import a nexus file, clean it up, correct for Mac/PC line breaks, etc.
sub ImportFile {
	$Mac = "\x{0D}";#chr(015);
	$PC = "\x{0A}";#chr(012);
	for ($i = 0; $i <= $#FileNexus; $i++) {
		$line = $FileNexus[$i];
		$line =~ s/\|/\|\|/g; #stops | being treated as a single whitespace

		$line =~ s/$Mac|$PC/\n/g;
		$line =~ tr/\n/\n/s;

		@TempList = split(/\n/, $line);
		for ($TT = 0; $TT <= $#TempList; $TT++){
			$TempLine = $TempList[$TT];
			unless(length($TempLine) == 0){
				push(@NexusList, $TempLine)
				}
			}
		}
	###################################
	#Take list and split it into a multidimensional array
	$LineCheck = 0;
	@TreeShape = ();
	$TreeInCount = 0;

	for ($i = 0; $i <= $#NexusList; $i++) {

		$OptLine = $NexusList[$i];
		if($OptLine =~ /Character-state optimization/){
			if($OptLine =~ /DELTRAN/){$Optimise = "DelTran"}
			elsif($OptLine =~ /ACCTRAN/){$Optimise = "AccTran"}
			else{$Optimise = "Unknown"}

			unless($Unambig eq "u"){
				print OUTPUT "\n$Optimise character-state optimization used";
				}
			}

		$Yap = $i;
		$TreeLine = $NexusList[$Yap];

		if($TreeLine =~ /Rescaled consistency index/){
			until(($TreeLine =~ / *\/-*/) or ($Yap > $#NexusList)){
				$Yap++;
				$TreeLine = $NexusList[$Yap];
				}
			if($Yap < $#NexusList){
				$TreeInCount = 0;
				until($TreeInCount == 1){
					$NextTreeLine = $NexusList[1 + $Yap];
					$TreeLine =~ s/\|\|/\|/g; #returns vertical lines to original form
					if(length($TreeLine) == 0){
						$Yap;
						$TreeLine = $NexusList[$Yap];
						next;
						}
					if($TreeLine =~ / *\\-*/){
						if(($TreeLine !~ /\|/) and ($TreeLine !~ /\\.+\\/) and ($TreeLine !~ /\//)){
							if(($NextTreeLine !~ /\\|\|/)){
								$TreeInCount = 1;
								}
							}
						}
					push(@TreeShape, $TreeLine);
					$Yap++;
					$TreeLine = $NexusList[$Yap];
					}
				#$TreeInCount = 2;
				}

			}

		$TheLine = $NexusList[$i];

		chomp $TheLine;
		$TheLine =~ tr/\n/\n/s;
		$TheLine =~ tr/A-Z/a-z/;
		$TheLine =~ s/  +/ /g;
		$TheLine =~ s/^ +//g;

		$TheLine =~ s/ --> node/ -I- node/g;
		$TheLine =~ s/node_(\d+) --> /node_$1 -I- /g;
		$TheLine =~ s/ ==> / u /g;
		$TheLine =~ s/ --> / a /g;

		@AddingLine1 = split(/\n/, $TheLine);


		foreach $Cut (@AddingLine1){

			if($Cut =~ /branch character steps ci change/){
				$LineCheck = 1;
				next
				}
			unless($LineCheck == 1){next}
			if($Cut =~ /\d\.\d\d\d/){
				@AddingLine2 = split(/ /, $Cut);
				#$" = "...."; #"
				#print ARSE "\n\@AddingLine2 is @AddingLine2";
				#$" = " "; #"
				#if($AddingLine2[1] eq "a"){$AddingLine2[1] = "-I-"}
				#elsif($AddingLine2[2] eq "a"){$AddingLine2[2] = "-I-"}
				push(@OriginalNEXUSstore, [@AddingLine2])
				}
			}
		}
	if($TreeInCount == 1){
		print OUTPUT "\n\nOriginal Tree:\n";
		foreach $TreeThing (@TreeShape){
			print OUTPUT "\n $TreeThing"
			}
		}
	} #end of ImportFile sub

#a sub to Import a nexus file, clean it up, correct for Mac/PC line breaks, etc.
####################################
####################################


#############################################################
#sub to locate and select a particular node
sub select_node{
	$MasterNodeCounter++;
	$NodeCounter = 0;

	for($Looking = 0; $Looking <= $#OriginalNEXUSstore; $Looking++){
		@temp = @{$OriginalNEXUSstore[$Looking]};
		if($#temp > 6){
			$NodeCounter++;
			if($NodeCounter == $MasterNodeCounter){
				&ReadInNode;
				}
			}
		}
	}
#end of sub to locate and select a particular node
#############################################################


#############################################################
#sub to read in data for a particular node
sub ReadInNode {
	@NodeInQuestion = ();
	$AmbORno = "";
	$LooperDooper = 0;
	$ThisorThat = 0;
	$ToNode = "";
	$FromNode = "";
	#print ARSE "\n\@temp is @temp";
	for($LineAlong = $#temp; $LineAlong >= 0 ; $LineAlong--){
		$LooperDooper++;
		if($LooperDooper == 1){$TempLine[2] = $temp[$LineAlong];}
		elsif($LooperDooper == 2){$AmbORno = $temp[$LineAlong];}
		elsif($LooperDooper == 3){$TempLine[1] = $temp[$LineAlong];}
		elsif($LooperDooper == 6){$TempLine[0] = $temp[$LineAlong];}

		if($LooperDooper > 6){
			if($temp[$LineAlong] eq "-I-"){
				$ThisorThat = 1;
				#print ARSE "\n\$ThisorThat is $ThisorThat";
				next
				}
			if($ThisorThat == 0){
				$ToNode = "$temp[$LineAlong]_" . $ToNode;
				#print ARSE "\n\$ToNode is $ToNode";
				}
			elsif($ThisorThat == 1){
				$FromNode = "$temp[$LineAlong]_" . $FromNode;
				#print ARSE "\n\$FromNode is $FromNode";
				}
			}
		}

	if($Unambig eq "u"){
			unless($AmbORno eq "a"){
				push(@NodeInQuestion, [@TempLine])
				}
			}
	else{push(@NodeInQuestion, [@TempLine])}

	#print ARSE "\n\@TempLine is @TempLine";

	chop($FromNode);
	chop($ToNode);
	$TempLiner = $FromNode . "-I-" . $ToNode;
	push(@MasterNodeList, $TempLiner);

	@TempLine = ();

	$Looking++;
	@temp = @{$OriginalNEXUSstore[$Looking]};
	until(($#temp > 6) or ($Looking > $#OriginalNEXUSstore)){
		@TempLine = ();
		if($Unambig eq "u"){
			if($temp[4] eq "a"){
				$Looking++;
				@temp = @{$OriginalNEXUSstore[$Looking]};
				next;
				}
			}
		$TempLine[0] = $temp[0];
		$TempLine[1] = $temp[3];
		$TempLine[2] = $temp[5];
		push(@NodeInQuestion, [@TempLine]);
		$Looking++;
		@temp = @{$OriginalNEXUSstore[$Looking]};
		}
	#print ARSE "\nRead in of $TempLiner";
	for($DigArse = 0; $DigArse <= $#NodeInQuestion; $DigArse++){
		@ArsetEMP = @{$NodeInQuestion[$DigArse]};
		#print ARSE "\n@ArsetEMP";
		}
	}

#end of sub to read in data for a particular node
#############################################################

######################################################
sub FindHighest {
	for($YY = 0; $YY <= $#NodeInQuestion; $YY++){
		@Bollocks = @{$NodeInQuestion[$YY]};
		$Test = $Bollocks[0];
		if($Test > $HighestChar){$HighestChar = $Test}
		}
	}
######################################################


#################################################
#sub to create lookup table
sub createlookup {
	print "\ncreating EP lookup table...\n";
	@Lookup = ();
	$HighChar = 0;
	$CharCounter = 0;
	$ColChar = 0;
	$RowChar = 1;
	until($CharCounter >= $HighestChar){
		$RowChar++;
		if($HighChar < $RowChar){$HighChar = $RowChar}
		$ColChar++;
		$ColClick = 0;
		while($ColClick < $ColChar){
			$ColClick++;
			$CharCounter++;
			$Input = $RowChar . "_" . $ColClick;
			push(@Lookup, $Input);
			}
		}
	}

#end of sub to create lookup table
#################################################


#################################################
#sub to convert EP chars to events and polarity
sub ConvertChars {
	for($TLC = 0; $TLC <= $#MasterNodeStore; $TLC++){#
		@Temp1 = @{$MasterNodeStore[$TLC]};#Temp1 = NodeInQuestion
		for($FLM = 0; $FLM <= $#Temp1; $FLM++){
			@Temp2 = @{$Temp1[$FLM]};#Temp1 = List for each char
			$EPChar = $Temp2[0];
			@Transform = split(/_/, $Lookup[$EPChar - 1]);
			splice(@Temp2, 0, 1, @Transform);
			$FromTest = $Temp2[2];
			$ToTest = $Temp2[3];
			if($FromTest > $ToTest){$Polarity = "E"}
			else{$Polarity = "L"}
			splice(@Temp2, 2, 2, $Polarity);
			$Temp1[$FLM] = [@Temp2];
			}
		$MasterNodeStore[$TLC] = [@Temp1];
		}
	}

#end of sub to convert EP chars to events and polarity
#################################################


##################################################################
#sub to get Assoc Array of Chars & states for a given node
sub ForCrackingList {
	@Temp1 = @{$MasterNodeStore[$ZZTop]};#Temp1 = NodeInQuestion
	$WorthIt = 0;
	if(@Temp1 > 0){$WorthIt = 1}

	%CrackingList = ();
	%LateList = ();
	%EarlyList = ();
	@RowCrack = (0, 0, [%LateList],[%EarlyList]);

	for($JJE = 1; $JJE <= $HighChar; $JJE++){
		$CrackingList{"$JJE"} = [@RowCrack];
		}

	for($SDP = 0; $SDP <= $#Temp1; $SDP++){
		@Temp2 = @{$Temp1[$SDP]};
		$RowChar = $Temp2[0];
		$ColChar = $Temp2[1];
		$RowDir = $Temp2[2];

		@RowCrack = @{$CrackingList{"$RowChar"}};

		if($RowDir eq "L"){
			%Temp3 = @{$RowCrack[2]};
			$Temp3{"$ColChar"} = "1";

			$RowCrack[0]++;
			$RowCrack[1]++;
			$RowCrack[2] = [%Temp3];

			%Temp4 = @{$CrackingList{"$ColChar"}[3]};
			$Temp4{"$RowChar"} = "1";
			$CrackingList{"$ColChar"}[3] = [%Temp4];
			$CrackingList{"$ColChar"}[0]--;
			$CrackingList{"$ColChar"}[1]++;
			}

		if($RowDir eq "E"){
			%Temp3 = @{$RowCrack[3]};
			$Temp3{"$ColChar"} = "1";
			$RowCrack[0]--;
			$RowCrack[1]++;
			$RowCrack[3] = [%Temp3];

			%Temp4 = @{$CrackingList{"$ColChar"}[2]};
			$Temp4{"$RowChar"} = "1";
			$CrackingList{"$ColChar"}[2] = [%Temp4];
			$CrackingList{"$ColChar"}[0]++;
			$CrackingList{"$ColChar"}[1]++;
			}
		$CrackingList{"$RowChar"} = [@RowCrack];
		}
	}

#end of sub to get Assoc Array of Chars & states for a given node
##################################################################


#############################################################
#Sub to scan and remove twins and add them to the assoc array %TempScore {"Twins"}
# Uses %InputArray for input
# Uses %TempScore to record output
sub Wipe_Twins {
	%TwinsTemp = @{$TempScore{"Twins"}};

	#Check for twins
	@tempkeys = keys(%InputArrayTwZo);
	foreach $ICE (@tempkeys){
		if($InputArrayTwZo{"$ICE"}[1] == 1){
			%LateTwin = @{$InputArrayTwZo{"$ICE"}[2]};
			%EarlyTwin = @{$InputArrayTwZo{"$ICE"}[3]};
			@EarlyKeys  = keys(%EarlyTwin);
			@LateKeys  = keys(%LateTwin);


			@ICEkeys = ();
			if(exists($TempScore{"$ICE"})){
				%ICEMove = @{$TempScore{"$ICE"}[1]};
				@ICEkeys = keys(%ICEMove);
				}

			unless(@ICEkeys > 0){

				#@EarlyKeys  = keys(%EarlyTwin);
				#@LateKeys  = keys(%LateTwin);

				if(@EarlyKeys == 1){

					@EarlyMoverkeys = ();
					if(exists($TempScore{"$EarlyKeys[0]"})){
						%EarlyMover = @{$TempScore{"$EarlyKeys[0]"}[1]};
						@EarlyMoverkeys = keys(%EarlyMover);
						}

					unless(@EarlyMoverkeys > 0){
						if($InputArrayTwZo{"$EarlyKeys[0]"}[1] == 1){

							#clear scores from ICE
							$InputArrayTwZo{"$ICE"}[3]= [()];
							$InputArrayTwZo{"$ICE"}[1]--;
							$InputArrayTwZo{"$ICE"}[0]--;
							delete($InputArrayTwZo{"$ICE"});

							#clear scores from TWIN
							$InputArrayTwZo{"$EarlyKeys[0]"}[2]= [()];
							$InputArrayTwZo{"$EarlyKeys[0]"}[1]--;
							$InputArrayTwZo{"$EarlyKeys[0]"}[0]++;
							delete($InputArrayTwZo{"$EarlyKeys[0]"});

							unless(exists($TwinsTemp{"$ICE"})){
								$TwinsTemp{"$ICE"} = "$EarlyKeys[0]";
								delete($InputArrayTwZo{"$ICE"});
								delete($InputArrayTwZo{"$EarlyKeys[0]"});
								$TempScore{"TotalScore"}++;  #print ARSE "(kill $ICE L $EarlyKeys[0]E)";
								if($TempScore{"TotalScore"} > $abortThreshold){$abort = 1};
								}
							}
						}
					}

				if(@LateKeys == 1){
					@LateMoverkeys = ();
					if(exists($TempScore{"$LateKeys[0]"})){
						%LateMover = @{$TempScore{"$LateKeys[0]"}[1]};
						@LateMoverkeys = keys(%EarlyMover);
						}

					unless(@LateMoverkeys > 0){
						if($InputArrayTwZo{"$LateKeys[0]"}[1] == 1){

							#clear scores from ICE
							$InputArrayTwZo{"$ICE"}[2]= [()];
							$InputArrayTwZo{"$ICE"}[1]--;
							$InputArrayTwZo{"$ICE"}[0]++;

							#clear scores from TWIN
							$InputArrayTwZo{"$LateKeys[0]"}[3]= [()];
							$InputArrayTwZo{"$LateKeys[0]"}[1]--;
							$InputArrayTwZo{"$LateKeys[0]"}[0]--;

							unless(exists($TwinsTemp{"$LateKeys[0]"})){
								$TwinsTemp{"$LateKeys[0]"} = "$ICE";
								delete($InputArrayTwZo{"$ICE"});
								delete($InputArrayTwZo{"$LateKeys[0]"});
								$TempScore{"TotalScore"}++; #print ARSE "(kill $ICE E $LateKeys[0]L)";
								if($TempScore{"TotalScore"} > $abortThreshold){$abort = 1};
								}
							}
						}
					}
				}
			}
		$TempScore{"Twins"} = [%TwinsTemp];
		}
	}

#Sub to scan and remove twins and add them to the assoc array %TempScore {"Twins"}
#############################################################


#############################################################
#Sub to scan and remove and zero-rated events
# Uses %InputArrayTwZo for input
sub Wipe_Zeros {
	@tempkeys = keys(%InputArrayTwZo);
	foreach $ICE (@tempkeys){
		if($InputArrayTwZo{"$ICE"}[1] == 0){
			unless(exists($TempScore{"$ICE"})){
				delete($InputArrayTwZo{"$ICE"});
				#print ARSE "\[Deleted $ICE zero\]";
				}
			}
		}
	}
#Sub to scan and remove and zero-rated events
#############################################################


#########################################################
#sub to find the best event for a dynamic-heuristic search
#uses data from %InputArrayBest
sub Find_Best {
	@HighestEvent = ();
	$HighestEventScore = 0;
	@Better = keys(%InputArrayBest);
	foreach $TMTP (@Better){
		if(abs($InputArrayBest{"$TMTP"}[0] * $InputArrayBest{"$TMTP"}[1]) == $HighestEventScore){
			push(@HighestEvent, $TMTP);
			}
		elsif(abs($InputArrayBest{"$TMTP"}[0] * $InputArrayBest{"$TMTP"}[1]) > $HighestEventScore){
			$HighestEventScore = abs(($InputArrayBest{"$TMTP"}[0] * $InputArrayBest{"$TMTP"}[1]));
			@HighestEvent = ();
			push(@HighestEvent, $TMTP);
			}
		}
	$TestEvent = $HighestEvent[int(rand($#HighestEvent + 1))];
	}
#end of sub to find the best event for a dynamic-heuristic search
#########################################################


############################################################
#sub to take an event and follow it through
#takes %input array and $TestEvent
sub Do_Event{
	#print ARSE " going with $TestEvent ";
	if((exists($DoInputArray{"$TestEvent"}))){
		push(@This_Doing_List, $TestEvent);
		$AnythingDoing = 0;
		$ButExists = 0;
		$PrePolarised = 0;
		#print ARSE " nontriv ";

		#get polarity
		if(exists($TempScore{"$TestEvent"})){
			$Polarity = $TempScore{"$TestEvent"}[0];
			%TempCharScore = @{$TempScore{"$TestEvent"}[1]};
			@ButorNut = keys(%TempCharScore);
			if(@ButorNut > 0){$ButExists = 1;}
			#print ARSE " polfile$Polarity ";
			}
		else{
			if($DoInputArray{"$TestEvent"}[0] < 0){$Polarity = "E"}
			elsif($DoInputArray{"$TestEvent"}[0] > 0){$Polarity = "L"}
			elsif($DoInputArray{"$TestEvent"}[0] == 0){
				@RandomPol = ('E', 'L');
				$Polarity = $RandomPol[int(rand($#RandomPol + 1))];
				}
			%TempCharScore = ();
			$TempScore{"$TestEvent"} = [$Polarity, [%TempCharScore]];
			#print ARSE " polset$Polarity ";
			}

		#Get early and late list;
		%TestEventLate = @{$DoInputArray{"$TestEvent"}[2]};
		%TestEventEarly = @{$DoInputArray{"$TestEvent"}[3]};
		$ConflictScore = 0;

		if($Polarity eq "L"){
			while(($LateKey, $LateValue) = each(%TestEventLate)){
				$TempCharScore{"$LateKey"} = "$LateValue";
				$AnythingDoing = 1;
				$DoInputArray{"$LateKey"}[1]--;
				$DoInputArray{"$LateKey"}[0]++;

				if($DoInputArray{"$LateKey"}[1] == 0){
					delete($DoInputArray{"$LateKey"});
					}
				else{
					%TempAdjust = @{$DoInputArray{"$LateKey"}[3]};
					delete($TempAdjust{"$TestEvent"});
					$DoInputArray{"$LateKey"}[3] = [%TempAdjust];
					}
				}

			if(($AnythingDoing == 1) and ($ButExists == 0)){
				$TempScore{"TotalScore"}++;
				if($TempScore{"TotalScore"} > $abortThreshold){$abort = 1}; #print ARSE "(New move $TestEvent L)";
				}
			else{
				#print ARSE "(L$TestEvent no $AnythingDoing, $ButExists)";
				}

			while(($EarlyKey, $EarlyValue) = each(%TestEventEarly)){
				@TempPolar = ();
				%TempMoveList = ();
				$ConflictScore = 0;
				$PrePolarised = 0;
				if(exists($TempScore{"$EarlyKey"})){
					if($TempScore{"$EarlyKey"}[0] ne "L"){
						@TempConflict = @{$TempScore{"Conflict"}};
						$TempString = $EarlyKey . "_E_" . $TestEvent . "_L";
						push(@TempConflict, $TempString);
						$TempScore{"Conflict"} = [@TempConflict];
						#print ARSE "(confl $TestEvent says $EarlyKey is L)";
						#%ExhaustCheck = @{$TempScore{"$EarlyKey"}[1]};
						#(%ExhaustCheck);
						$ConflictScore = 1;
						}
					if($TempScore{"$EarlyKey"}[2] eq "P"){$PrePolarised = 1};
					@TempPolar = @{$TempScore{"$EarlyKey"}};
					%TempMoveList = @{$TempPolar[1]};
					}

				if($ConflictScore == 1){
					%TempAdjust = @{$DoInputArray{"$EarlyKey"}[2]};
					delete($TempAdjust{"$TestEvent"});
					$DoInputArray{"$EarlyKey"}[2] = [%TempAdjust];
					$DoInputArray{"$EarlyKey"}[1] --;
					$DoInputArray{"$EarlyKey"}[0] --;
					$TempPolar[0] = "E";
					}
				else{
					%TempAdjust = @{$DoInputArray{"$EarlyKey"}[2]};
					delete($TempAdjust{"$TestEvent"});
					$DoInputArray{"$EarlyKey"}[2] = [%TempAdjust];
					$DoInputArray{"$EarlyKey"}[1] --;
					$DoInputArray{"$EarlyKey"}[0] --;
					$TempPolar[0] = "L";


					unless((exists($TempScore{"$EarlyKey"})) and ($PrePolarised == 0)){
						$TempScore{"TotalScore"}++; #print ARSE "(New pol $EarlyKey of $TestEvent)";
						}
					}

				unless(exists($TempScore{"$EarlyKey"})){
					if($WhichExhaust <= $#TempGuide){
						push(@TempGuide, $EarlyKey)
						}
					else{$TempGuide[$WhichExhaust + 1] = $EarlyKey}
					$AdjustedGuide = 1;
					};

				if($ConflictScore == 0){
					$TempMoveList{"$TestEvent"} = "1";
					}
				$TempPolar[1] = [%TempMoveList];
				$TempScore{"$EarlyKey"} = [@TempPolar];
				if($TempScore{"TotalScore"} > $abortThreshold){$abort = 1};
				}
			}

		if($Polarity eq "E"){
			while(($EarlyKey, $EarlyValue) = each(%TestEventEarly)){
				$TempCharScore{"$EarlyKey"} = "$EarlyValue";
				$AnythingDoing = 1;

				$DoInputArray{"$EarlyKey"}[1]--;
				$DoInputArray{"$EarlyKey"}[0]--;

				if($DoInputArray{"$EarlyKey"}[1] == 0){
					delete($DoInputArray{"$EarlyKey"});
					}
				else{
					%TempAdjust = @{$DoInputArray{"$EarlyKey"}[2]};
					delete($TempAdjust{"$TestEvent"});
					$DoInputArray{"$EarlyKey"}[2] = [%TempAdjust];
					}
				}

			if(($AnythingDoing == 1) and ($ButExists == 0)){
				$TempScore{"TotalScore"}++;  #print ARSE "(New move $TestEvent E)";
				if($TempScore{"TotalScore"} > $abortThreshold){$abort = 1};
				}
			else{
				#print ARSE "(E$TestEvent no $AnythingDoing, $ButExists)";
				}

			while(($LateKey, $LateValue) = each(%TestEventLate)){
				@TempPolar = ();
				%TempMoveList = ();
				$ConflictScore = 0;
				$PrePolarised = 0;
				if(exists($TempScore{"$LateKey"})){
					if($TempScore{"$LateKey"}[0] ne "E"){
						@TempConflict = @{$TempScore{"Conflict"}};
						$TempString = $LateKey . "_L_" . $TestEvent . "_E";
						push(@TempConflict, $TempString);
						$TempScore{"Conflict"} = [@TempConflict];
						$ConflictScore = 1;
						#print ARSE "(confl $TestEvent says $LateKey is E)";
						}

					if($TempScore{"$LateKey"}[2] eq "P"){$PrePolarised = 1};
					@TempPolar = @{$TempScore{"$LateKey"}};
					%TempMoveList = @{$TempPolar[1]};
					}

				if($ConflictScore == 1){
					%TempAdjust = @{$DoInputArray{"$LateKey"}[3]};
					delete($TempAdjust{"$TestEvent"});
					$DoInputArray{"$LateKey"}[3] = [%TempAdjust];
					$DoInputArray{"$LateKey"}[1] --;
					$DoInputArray{"$LateKey"}[0] ++;
					$TempPolar[0] = "L";
					}

				else{
					%TempAdjust = @{$DoInputArray{"$LateKey"}[3]};
					delete($TempAdjust{"$TestEvent"});
					$DoInputArray{"$LateKey"}[3] = [%TempAdjust];
					$DoInputArray{"$LateKey"}[1] --;
					$DoInputArray{"$LateKey"}[0] ++;
					$TempPolar[0] = "E";
					unless((exists($TempScore{"$LateKey"})) and ($PrePolarised == 0)){
						$TempScore{"TotalScore"}++;  #print ARSE "(New pol $LateKey of $TestEvent)";
						}
					}

				unless(exists($TempScore{"$LateKey"})){
					if($WhichExhaust <= $#TempGuide){
						push(@TempGuide, $LateKey)
						}
					else{$TempGuide[$WhichExhaust + 1] = $LateKey}
					$AdjustedGuide = 1;
					};

				if($ConflictScore == 0){
					$TempMoveList{"$TestEvent"} = "1";
					}
				$TempPolar[1] = [%TempMoveList];
				$TempScore{"$LateKey"} = [@TempPolar];
				if($TempScore{"TotalScore"} > $abortThreshold){
					$abort = 1;
					}
				}
			}
		$TempScore{"$TestEvent"}[1] = [%TempCharScore];
		}
	delete($DoInputArray{"$TestEvent"});
	#print ARSE " done $TestEvent ";
	}

#end of sub to take an event and follow it through
############################################################


##################################################
#temp sub to do a single complete run on one node
#takes %InputArray, @CycleLength
sub Do_Node {
	@CycleLength = keys(%NodeInputArray);
	@This_Doing_List = ();

	while(@CycleLength){
		if($abort == 1){last}
		$WhichExhaust++;
		%InputArrayTwZo = %NodeInputArray;
		&Wipe_Twins;
		&Wipe_Zeros;
		%NodeInputArray = %InputArrayTwZo;
		@CycleLength = keys(%NodeInputArray);

		if(@CycleLength == 0){last};

		if($HSearch == 1){
			%InputArrayBest = %NodeInputArray;
			&Find_Best;
			}

		elsif($HSearch == 2){
			if($WhichExhaust <= $#TempGuide){$TestEvent = $TempGuide[$WhichExhaust];}

			else{
				%InputArrayBest = %NodeInputArray;
				&Find_Best;
				};
			}

		%DoInputArray = %NodeInputArray;
		&Do_Event;
		%NodeInputArray = %DoInputArray;
		%PrintArrayTemp = %NodeInputArray;
		@CycleLength = keys(%NodeInputArray);
		}

	if(($TempScore{"TotalScore"} == $abortThreshold) and ($abort == 0)){
		if($Wherethefuck == 2){
			if($AdjustedGuide == 1){print DATA " \(adjusted to @TempGuide\).\nFinal order used @This_Doing_List\n"}
			elsif($AdjustedGuide == 0){print DATA "\nFinal order used @This_Doing_List\n"}
			}
		#print ARSE "\nDid print something 'cause ". $TempScore{"TotalScore"} . " = $abortThreshold and $abort is 0";
		if($Wherethefuck == 1){
			if(@This_Doing_List > 0 ){print DATA "Final order used is @This_Doing_List\n";}
				else{print DATA "Final order null (no nontrivial seeds)\n"}
			}
		&Print_Final;
		}
	else{
		#print ARSE "\nDidn't print anything 'cause ". $TempScore{"TotalScore"} . " didn't = $abortThreshold or $abort is 1";
		}
	}
#################################################



#########################################################
#sub to look for likely lowest score, to use as a threshold for exhaustive searches
#takes %inputarray and @CycleLength, gives out $abortThreshold (returns input array to original)
sub ThresholdSearch {
	@TempCycleLength = keys(%InputArrayThresh);
	$abortThreshold = @TempCycleLength;
	#print ARSE "\nStarting with \$abortThreshold at $abortThreshold";

	while(@TempCycleLength){

		%InputArrayTwZo = %InputArrayThresh;
		&Wipe_Twins;
		&Wipe_Zeros;
		%InputArrayThresh = %InputArrayTwZo;

		@TempCycleLength = keys(%InputArrayThresh);

		if(@TempCycleLength == 0){last};

		%InputArrayBest = %InputArrayThresh;
		&Find_Best;

		%DoInputArray = %InputArrayThresh;
		&Do_Event;
		%InputArrayThresh =  %DoInputArray;

		@TempCycleLength = keys(%InputArrayThresh);

		}

	@ConflictLength = @{$TempScore{"Conflict"}};
	if(@ConflictLength == 0){
		$abortThreshold = $TempScore{"TotalScore"};
		#print ARSE "\n No conflicts so \$abortThreshold is $abortThreshold";
		}
	}


#########################################################
#sub to look for likely lowest score, to use as a threshold for exhaustive searches


#################################################
#sub to print to TEMP file final data of the run
sub Print_Final {
	@TempNodePrint = split(/-I-/, $MasterNodeList[$ZZTop]);
	$Temping = $TempScore{"TotalScore"};
	print DATA "Score $Temping\n";

	%TwinsTemp = @{$TempScore{"Twins"}};
	@TempKeys = sort{$a <=> $b}(keys(%TwinsTemp));
	print DATA "Twins";
	foreach $ABC (@TempKeys){
		$DEF = $TwinsTemp{"$ABC"};
		print DATA " \( $ABC $DEF \)";
		}
	print DATA "\n";

	@TempMove = sort{$a <=> $b}(keys(%TempScore));

	foreach $PrintChar (@TempMove){
		unless(($PrintChar eq "Twins") or ($PrintChar eq "TotalScore") or ($PrintChar eq "Conflict") or (@{$TempScore{"$PrintChar"}[1]} == 0)){
			%PrintTemp = @{$TempScore{"$PrintChar"}[1]};
			$PolTemp = $TempScore{"$PrintChar"}[0];
			print DATA "Char $PrintChar moved $PolTemp relative to ";
			@TempList = sort{$a <=> $b}(keys(%PrintTemp));
			print DATA "@TempList\n";
			}
		}

	@temp = @{$TempScore{"Conflict"}};
	if(@temp){
		print DATA "Conflicts of polarity";
		foreach $PSV (@temp){
			print DATA " \( $PSV \)";
			}
		print DATA "\n";
		}
	print DATA "\n";
	}

#end of sub to print to TEMP file final data
#################################################





#################################################
#Sub to read-in and assess data from the current node's file
sub ReadInCrackData {
	@CrackData = ();
	open(NODEDATA,  '<', $ReadInFile) or die "\nThe file $ReadInFile could not be found.\n";
	#print BUM "\nMade it to ReadInCrackData, opened $ReadInFile";
	@CrackData = <NODEDATA>;
	#print BUM "\n$#CrackData is " . $#CrackData . "OK?";
	$PrimeRun = 0;
	$ConfPrimeRun = 0;
	$WhatRun = 0;
	%CompareList = ();


	$Previous = "Conflicts";
	$ConflictTest = 0;
	$OnlyConflicts = 1;
	foreach $ConflTemp(@CrackData){
		chomp($ConflTemp);
		if($ConflTemp =~ /Conflicts/){$ConflictTest = 1}
		if(length($ConflTemp) == 0){
			if($Previous !~ /Conflicts/){$OnlyConflicts = 0}
			}
		$Previous = $ConflTemp;
		}
	#print BUM "\nI think \$ConflictTest $ConflictTest \$OnlyConflicts $OnlyConflicts";

	for($CrackLine = 0; $CrackLine <= $#CrackData; $CrackLine++){
		$Shorter = 0;
		chomp($CrackData[$CrackLine]);
		@LineSplit = split(/ /, $CrackData[$CrackLine]);
		#$" = "-!-"; #"
		#print BUM "\n$CrackLine is @LineSplit, so \'$LineSplit[0]\'";
		#$" = " "; #"

		if($LineSplit[0] eq "New"){
			#print BUM "\n$LineSplit[0]";
			next
			}
		elsif(($LineSplit[0] eq "Guided") or ($LineSplit[0] eq "Free")){
			$GuidedorFree = "F";
			if($LineSplit[0] eq "Guided"){$GuidedorFree = "G"}
			#print BUM "\n\$GuidedorFree is $GuidedorFree";

			@TempNextLine = split(/ /, $CrackData[$CrackLine + 1]);
			#print BUM "\n\@TempNextLine is \'@TempNextLine\'";
			if($TempNextLine[0] eq "Final"){
				#$" = "-!-"; #"
				#print BUM "\n$TempNextLine[0] is Final";
				#$" = " "; #"

				&CompareCrack;
				next}
			elsif($TempNextLine[0] eq "--------------"){
				#print BUM "\n$TempNextLine[0] is --------";
				$Shorter = 1;
				&CompareCrack;
				}
			else{
				#print BUM "\n$TempNextLine[0] is something";
				next
				}
			}

		elsif($LineSplit[0] eq "--------------"){
			$Shorter = 1;
			$CrackLine++;
			chomp($CrackData[$CrackLine]);
			@TempRunLine = split(/ /, $CrackData[$CrackLine]);
			&CompareCrack;
			}
		elsif($LineSplit[0] eq "End"){next}
		elsif($LineSplit[0] =~ /========/){&RecordArray; last}
		else{
			#print BUM " hopelessly lost on line $CrackLine "
			}
		}
	close(NODEDATA);
	if($DeleteOldFile eq 'y'){unlink($ReadInFile)}

	}
#end of sub to read-in and assess data from the current node's file
#################################################



#################################################################
#sub to filter the runs, and send them to the correct compare sub
sub CompareCrack{
	$WhatRun = $LineSplit[2];
	if($WhatRun =~ /,/){chop($WhatRun)};#remove comma
	$WhatRun = $GuidedorFree . "_" . $WhatRun;

	#$LineCounter = 0;
	$CrackLine++;
	chomp($CrackData[$CrackLine]);
	@TempRunLine = split(/ /, $CrackData[$CrackLine]);
	#print BUM "at Comp Crack, \@TempRunLine is @TempRunLine";

	$AreThereConflicts = 0;
	if($ConflictTest == 1){
		#print BUM "and \$ConflictTest is $ConflictTest";
		$ConflictLine = $CrackLine;
		$AreThereConflicts = 0;
		@TempConflictLine = (1,2);
		until(@TempConflictLine == 0){
			$ConflictLine++;
			@TempConflictLine = split(/ /, $CrackData[$ConflictLine]);
			if($TempConflictLine[0] eq "Conflicts"){
				$AreThereConflicts = 1;
				#print BUM ", \$AreThereConflicts is $AreThereConflicts";
				}
			};

		if($Shorter == 1){
			#print BUM ", \$Shorter is $Shorter";
			$CrackLine++;
			chomp($CrackData[$CrackLine]);
			@LineSplit = split(/ /, $CrackData[$CrackLine]);
			}
		}

	if($AreThereConflicts == 1){
		$LineCounter=0;
		#print BUM "\nGoing to \&ConflictCompare";
		&ConflictCompare;
		$ConfPrimeRun = 1;
		$LineCounter=0;
		}
	else{
		$LineCounter=0;
		#print BUM "\nGoing to \&NonConflictCompare";
		&NonConflictCompare;
		$PrimeRun = 1;
		$LineCounter=0;

		}
	}
#end of sub to filter the runs, and send them to the correct compare sub
#################################################################


########################################################
#sub to look at runs without conflicts
sub NonConflictCompare {
	%FinalRunTwins = ();
	%FinalRunFinal = ();
	%CrackMoveList = ();
	#print BUM "\n\@TempRunLine is @TempRunLine";
	until(@TempRunLine == 0){
		$LineCounter++;
		#print BUM "\n\$LineCounter is $LineCounter";
		if($LineCounter == 1){
			#print BUM "\n\$PrimeRun $PrimeRun";
			if($PrimeRun == 0){
				$CrackLine++;
				@TempRunLine = split(/ /, $CrackData[$CrackLine]);
				$Score = $TempRunLine[1];
				#print BUM "\nTaking score $Score from @TempRunLine \n";
				$CrackLine++;
				@TempRunLine = split(/ /, $CrackData[$CrackLine]);
				}
			else{
				$CrackLine += 2;
				@TempRunLine = split(/ /, $CrackData[$CrackLine]);
				}
			}#score input if prime run

		elsif($LineCounter == 2){
			for($MoveAlong = 1; $MoveAlong <= $#TempRunLine; $MoveAlong++){
				if($TempRunLine[$MoveAlong] eq '('){
					$MoveAlong++;
					$Latesome = $TempRunLine[$MoveAlong];
					$MoveAlong++;
					$Earlysome = $TempRunLine[$MoveAlong];
					$MoveAlong ++;
					$FinalRunTwins{"$Latesome"} = "$Earlysome";
					}
				}

			$FinalRunFinal{"Twins"} = [%FinalRunTwins];
			$CrackLine++;
			@TempRunLine = split(/ /, $CrackData[$CrackLine]);
			}#twins input

		elsif($LineCounter > 2){
			%CrackMoveList = ();
			$CrackPolTemp = $TempRunLine[3];

			for($InputTemp = 6; $InputTemp <= $#TempRunLine; $InputTemp++){
				$CrackMoveList{"$TempRunLine[$InputTemp]"} = "1";
				}

			$FinalRunFinal{"$TempRunLine[1]"} = [$CrackPolTemp, [%CrackMoveList]];
			$CrackLine++;
			@TempRunLine = split(/ /, $CrackData[$CrackLine]);
			}#char input
		}

	if($PrimeRun == 0){
		$CompareListFinal{"$WhatRun"} = [%FinalRunFinal];
		%Consensus = ();
		%Consensus = %FinalRunFinal;
		$PrimeRun = 1;
		}

	else{
		%ActualCon = ();
		%ActualCon = %Consensus;
		%ActualFinal = ();
		%ActualFinal = %FinalRunFinal;
		%ActualCompareTotal = ();
		%ActualCompareTotal = %CompareListFinal;
		$ConforNorm = 0;
		&ActuallyCompare;
		%CompareListFinal = %ActualCompareTotal;
		%Consensus = %ActualCon;
		}
	}

#end of sub to look at runs without conflicts
########################################################


########################################################
#sub to compare runs and add them to the comparelist if necessary
#Thought - to sort conflicting data - I'd have to have std input hashes
#and create a list of comparers for conflicting
#and add a bit to check length - or not, as the charlist and twinlist would be different
sub ActuallyCompare {
	@CompareKeys = keys(%ActualCompareTotal);
	$AddorNot = 0;
	@AddorNotList = ();
	$FoundtheSameOne = 0;

	foreach $Comparecycle (@CompareKeys){
		%TempCycle = ();
		%TempTwins = ();
		%FRFTwins = ();

		%TempCycle = @{$ActualCompareTotal{"$Comparecycle"}};
		%TempTwins =  @{$TempCycle{"Twins"}};
		@TempTwinKeys = keys(%TempTwins);
		@TempCycleKeys = keys(%TempCycle);
		@FRFKeys = keys(%ActualFinal);
		%FRFTwins = @{$ActualFinal{"Twins"}};
		@FRFTwinKeys = keys(%FRFTwins);

		if($ConforNorm == 1){
			$TempCycleLength = $TempCycle{"Score"};
			$TempFinalLength = $ActualFinal{"Score"};
			if($TempFinalLength != $TempCycleLength){
				$AddorNot = 1;
				push(@AddorNotList, $AddorNot);
				$AddorNot = 0;
				next;
				}

			%ConflictFinal = @{$TempCycle{"Conflicts"}};
			%ConflictCycle = @{$ActualFinal{"Conflicts"}};
			@FinalCompKeys = keys(%ConflictFinal);
			foreach $KeyCheck (@FinalCompKeys){
				if(exists($ConflictCycle{"$KeyCheck"})){
					delete($ConflictCycle{"$KeyCheck"})
					}
				else{
					$AddorNot = 1;
					push(@AddorNotList, $AddorNot);
					$AddorNot = 0;
					next;
					}
				}
			@FinalCompKeys = keys(%ConflictCycle);
			if(@FinalCompKeys > 0){
				$AddorNot = 1;
				push(@AddorNotList, $AddorNot);
				$AddorNot = 0;
				next;
				}
			}

		foreach $Tester (@FRFTwinKeys){
			if(exists($TempTwins{"$Tester"})){
				if($FRFTwins{"$Tester"} == $TempTwins{"$Tester"}){
					delete($TempTwins{"$Tester"});
					}
				else{
					$AddorNot = 1;
					%TwinsCut = @{$ActualCon{"Twins"}};
					delete($TwinsCut{"$Tester"});
					$ActualCon{"Twins"} = [%TwinsCut];
					delete($TempTwins{"$Tester"});
					}
				}

			else{
				$AddorNot = 1;
				%TwinsCut = @{$ActualCon{"Twins"}};
				delete($TwinsCut{"$Tester"});
				$ActualCon{"Twins"} = [%TwinsCut];
				}
			}

		@TempTwinKeys = keys(%TempTwins);
		if(@TempTwinKeys > 0){
			foreach $Tester (@TempTwinKeys){
				%TwinsCut = @{$ActualCon{"Twins"}};
				delete($TwinsCut{"$Tester"});
				$ActualCon{"Twins"} = [%TwinsCut];
				}
			$AddorNot = 1;
			}
		delete($TempCycle{"Twins"});

		foreach $Tester (@FRFKeys){
			unless($Tester eq "Twins"){
				if(exists($TempCycle{"$Tester"})){
					if($ActualFinal{"$Tester"}[0] eq $TempCycle{"$Tester"}[0]){
						%FRFMoveList = ();
						%TempCycleMoveList = ();
						%FRFMoveList = @{$ActualFinal{"$Tester"}[1]};
						@DeleteThis = keys(%FRFMoveList);
						@DeleteThis = ();
						@FRFMoveListKeys = keys(%FRFMoveList);
						%TempCycleMoveList = @{$TempCycle{"$Tester"}[1]};
						@DeleteThis = keys(%TempCycleMoveList);

						foreach $MoveTester (@FRFMoveListKeys){
							if(exists($TempCycleMoveList{"$MoveTester"})){
								delete($TempCycleMoveList{"$MoveTester"});
								}
							else{
								$AddorNot = 1;
								if(exists($ActualCon{"$Tester"})){
									%MoverCut = @{$ActualCon{"$Tester"}[1]};
									delete($MoverCut{"$MoveTester"});
									$ActualCon{"$Tester"}[1] = [%MoverCut];
									}
								delete($TempCycleMoveList{"$MoveTester"});
								}
							}
						delete($TempCycle{"$Tester"});
						}

					else{
						$AddorNot = 1;
						delete($ActualCon{"$Tester"});
						delete($TempCycle{"$Tester"});
						}
					}

				else{
					$AddorNot = 1;
					delete($ActualCon{"$Tester"});
					}
				}
			}#end of event check

		@TempCycleKeys = keys(%TempCycle);
		if(@TempCycleKeys > 0){
			foreach $Tester (@TempCycleKeys){
				delete($ActualCon{"$Tester"});
				}
			$AddorNot = 1;
			}

		#push(@AddorNotList, $AddorNot);
		if($AddorNot == 0){$FoundtheSameOne = 1; last};
		$AddorNot = 0;
		}#end of the round of comparison

	if($FoundtheSameOne == 0){
		$ActualCompareTotal{"$WhatRun"} = [%ActualFinal];
		$AddorNot = 0;
		}

	#$AddorNotResult = grep(/0/, @AddorNotList);
	#if($AddorNotResult == 0){
	#	$ActualCompareTotal{"$WhatRun"} = [%ActualFinal];
	#	$AddorNot = 0;
	#	}
	}
#end of sub to compare runs and add them to the comparelist if necessary
########################################################


########################################################
#sub to input a run with conflicts
sub ConflictCompare {
	%FinalRunTwinsConf = ();
	%FinalRunConflict = ();
	%CrackMoveListConf = ();

	until(@TempRunLine == 0){
		$LineCounter++;

		if($LineCounter == 1){
			$CrackLine++;
			@TempRunLine = split(/ /, $CrackData[$CrackLine]);
			$ConfScore = $TempRunLine[1];
			$FinalRunConflict{"Score"} = "$ConfScore";
			$CrackLine++;
			@TempRunLine = split(/ /, $CrackData[$CrackLine]);
			}#score input

		elsif($LineCounter == 2){
			for($MoveAlong = 1; $MoveAlong <= $#TempRunLine; $MoveAlong++){
				if($TempRunLine[$MoveAlong] eq '('){
					$MoveAlong++;
					$Latesome = $TempRunLine[$MoveAlong];
					$MoveAlong++;
					$Earlysome = $TempRunLine[$MoveAlong];
					$MoveAlong ++;
					$FinalRunTwinsConf{"$Latesome"} = "$Earlysome";
					}
				}
			$FinalRunConflict{"Twins"} = [%FinalRunTwinsConf];
			$CrackLine++;
			@TempRunLine = split(/ /, $CrackData[$CrackLine]);
			}#twins input

		elsif($LineCounter > 2){
			if($TempRunLine[0] eq "Char"){
				%CrackMoveListConf = ();
				$CrackPolTemp = $TempRunLine[3];

				for($InputTemp = 6; $InputTemp <= $#TempRunLine; $InputTemp++){
					$CrackMoveListConf{"$TempRunLine[$InputTemp]"} = "1";
					}
				$FinalRunConflict{"$TempRunLine[1]"} = [$CrackPolTemp, [%CrackMoveListConf]];
				}#char input

			elsif($TempRunLine[0] eq "Conflicts"){
				%ConflictSurvey = ();
				for($MoveAlong = 3; $MoveAlong <= $#TempRunLine; $MoveAlong++){
					if($TempRunLine[$MoveAlong] eq '('){
						$MoveAlong++;
						$ConflictFound = $TempRunLine[$MoveAlong];
						$MoveAlong += 2;
						$ConflictSurvey{"$ConflictFound"} = '1';
						}
					}
				}#conflict input

			$CrackLine++;
			@TempRunLine = split(/ /, $CrackData[$CrackLine]);
			}#char input

		$FinalRunConflict{"Conflicts"} = [%ConflictSurvey];
		}
	#$ConflictingRuns{"$WhatRun"} = [%FinalRunConflict];


	if($ConfPrimeRun == 0){
		$ConflictUnique{"$WhatRun"} = [%FinalRunConflict];
		}
	else{
		%ActualCon = ();
		%ActualFinal = %FinalRunConflict;
		%ActualCompareTotal = ();
		%ActualCompareTotal = %ConflictUnique;
		$ConforNorm = 1;
		&ActuallyCompare;
		%ConflictUnique = %ActualCompareTotal;
		}
	}
#end of sub to input a run with conflicts
########################################################


######################################################
#sub to print out the lot
sub PrintOutAll {
	#print consensus to STORE...
	@FindtheLength = keys(%CompareListFinal);
	$HowManyRuns = @FindtheLength;
	$plural = " ";
	if($HowManyRuns > 1){$plural = "s ";}
	%ConsensusTwins = @{$Consensus{"Twins"}};
	@LengthTwinKey = keys(%ConsensusTwins);
	@ConsensusMove = keys(%Consensus);
	$NoofSteps = (1 + $#LengthTwinKey + $#ConsensusMove);

	$PercentLength = (($NoofSteps/$Score)*100);

	if((@LengthTwinKey > 0) or (@ConsensusMove > 1)){
		print STORE "\n======================================";
		#print STORE "\n$TempLookList[2] characters with informative movement\n$TempLookList[0] ($TempLookList[1] iterations)\n";
		print STORE "\nConsensus of $HowManyRuns MS solution" . $plural . "without conflicts\nConsensus has $NoofSteps\/$Score steps \(";
		printf STORE ("%.1f", $PercentLength);
		print STORE "\%\)\n";

		%PrintHash = %Consensus;
		$whereto = "STORE";
		&PrintOutConsensus;
		}

	else{
		print STORE "\n\n======================================";
		print STORE "\n$FinalFromTo[0] --> $FinalFromTo[1]";
		print STORE "\n$TempLookList[2] characters with informative movement\n$TempLookList[0] ($TempLookList[1] iterations)\n";
		print STORE "\nConsensus of $HowManyRuns MS solution" . $plural . "without conflicts has no steps";
		}

	#print consensus to OUTPUT...
	%ConsensusTwins = @{$Consensus{"Twins"}};
	@LengthTwinKey = keys(%ConsensusTwins);
	@ConsensusMove = keys(%Consensus);

	if((@LengthTwinKey > 0) or (@ConsensusMove > 1)){
		print OUTPUT "\n\n======================================";
		print OUTPUT "\n$FinalFromTo[0] --> $FinalFromTo[1]";
		print OUTPUT "\n$TempLookList[2] characters with informative movement\n$TempLookList[0] ($TempLookList[1] iterations)\n";
		print OUTPUT "\nConsensus of $HowManyRuns MS solution" . $plural . "without conflicts\nConsensus has $NoofSteps\/$Score steps \(";
		printf OUTPUT ("%.1f", $PercentLength);
		print OUTPUT "\%\)\n";

		%PrintHash = %Consensus;
		$whereto = "OUTPUT";
		&PrintOutConsensus;
		}
	else{
		print OUTPUT "\n\n======================================";
		print OUTPUT "\n$FinalFromTo[0] --> $FinalFromTo[1]";
		print OUTPUT "\n$TempLookList[2] characters with informative movement\n$TempLookList[0] ($TempLookList[1] iterations)\n";
		print OUTPUT "\nConsensus of $HowManyRuns MS solution" . $plural . "without conflicts has no steps";
		}

	#print MS runs to STORE...
	$whereto = "STORE";
	if($OnlyConflicts == 1){
		print STORE "\n\n======================================";
		print STORE "\nAll MS runs recovered have conflicts\n";
		}
	else{
		@PrintOutKeys = sort{&Sorter}(keys(%CompareListFinal));
		$HowManyRuns = @PrintOutKeys;
		$plural = " ";
		if($HowManyRuns > 1){$plural = "s ";}
		$plural2 = "";
		if($Score > 1){$plural2 = "s";};

		print STORE "\n\n\n======================================";
		print STORE "\nData for $HowManyRuns MS run" . $plural . "without conflicts \($Score step$plural2\):";

		$PrintOutLoop = 0;

		foreach $PrintOutTempy (@PrintOutKeys){
			$PrintOutLoop++;
			%PrintHash = @{$CompareListFinal{"$PrintOutTempy"}};

			@PrintSearchType = split(/_/, $PrintOutTempy);
			if($PrintSearchType[0] eq "G"){$RunTypeFinal = "Guided"}
			elsif($PrintSearchType[0] eq "F"){$RunTypeFinal = "Free"};

			print STORE "\n\n$PrintOutLoop. Solution first identified in $RunTypeFinal run number $PrintSearchType[1]:";
			&PrintOutConsensus;
			print STORE "\n---------------------";
			}
		}

	#print conflict runs to STORE...
	if($ConflictTest == 1){
		@PrintOutKeys = sort{&Sorter}(keys(%ConflictUnique));
		$HowManyRuns = @PrintOutKeys;
		$plural = " ";
		if($HowManyRuns > 1){$plural = "s ";}
		print STORE "\n\n\n======================================";
		print STORE "\nData for $HowManyRuns run" . $plural . "with conflicts:";

		$PrintOutLoop = 0;
		foreach $PrintOutTempy (@PrintOutKeys){
			$PrintOutLoop++;
			%PrintHash = @{$ConflictUnique{"$PrintOutTempy"}};

			@PrintSearchType = split(/_/, $PrintOutTempy);
			if($PrintSearchType[0] eq "G"){$RunTypeFinal = "Guided"}
			elsif($PrintSearchType[0] eq "F"){$RunTypeFinal = "Free"};

			print STORE "\n\n$PrintOutLoop. Solution first identified in $RunTypeFinal run number $PrintSearchType[1]:";
			&PrintOutConsensus;
			print STORE "\n---------------------";

			}
		}

	else{
		print STORE "\n\n======================================";
		print STORE "\nNo runs with conflicts";
		print STORE "\n======================================";
		}

	print STORE "\n\n";
	print STORE "\n\n===========================================\n";
	for($FNFAL = 0; $FNFAL <= $#ArrayPrintOut; $FNFAL++){
		$TempLine = $ArrayPrintOut[$FNFAL];
		print STORE "$TempLine\n";
		}
	}
#end of sub to print out the lot
######################################################


######################################################
#sub to print out a given sequence
sub PrintOutConsensus {
	%PrintTwins = @{$PrintHash{"Twins"}};
	@PrintTwinsKeys = sort{$a <=> $b}(keys(%PrintTwins));
	delete($PrintHash{"Twins"});

	$printconflict = 0;
	if(exists($PrintHash{"Conflicts"})){
		$ConfScore = $PrintHash{"Score"};
		delete($PrintHash{"Score"});
		%PrintConfTemp = @{$PrintHash{"Conflicts"}};
		delete($PrintHash{"Conflicts"});
		$printconflict = 1;
		}

	@SmallPrintKeys = sort{$a <=> $b}(keys(%PrintHash));
	$TwinsYaNa = 0;
	$CharYaNa = 0;

	if($printconflict == 1){
		if($ConfScore < $Score){
			print $whereto "\nScore is $ConfScore \(\*shorter than MS score\*\)";
			}
		else{print $whereto "\nScore is $ConfScore"}
		}

	if(@PrintTwinsKeys > 0){
		print $whereto "\nTwins";
		foreach $Tempy (@PrintTwinsKeys){
			print $whereto " \($Tempy, " . $PrintTwins{"$Tempy"} . "\)";
			}
		$TwinsYaNa = 1;
		}

	if(@SmallPrintKeys > 0){
		$" = ", ";#"
		foreach $Tempy (@SmallPrintKeys){
			$PrintPol = $PrintHash{"$Tempy"}[0];
			%PrintCharFinal = @{$PrintHash{"$Tempy"}[1]};

			@PrintMovers = sort{$a <=> $b}(keys(%PrintCharFinal));

			print $whereto "\nChar $Tempy moved $PrintPol relative to @PrintMovers";
			}
		$CharYaNa = 1;
		$" = " ";#"
		}

	if($printconflict == 1){
		@ConfKeys = sort{$a cmp $b}(keys(%PrintConfTemp));
		print $whereto "\n\nConflicts of polarity:";
		foreach $ConfSur (@ConfKeys){
			@PrintSplit = split(/_/, $ConfSur);
			print $whereto " \($PrintSplit[2] says $PrintSplit[0] is $PrintSplit[3]\)";
			}
		}
	}
#end of sub to print out a given sequence
######################################################


########################################
#sub to sort final print list
sub Sorter {
	@X = split(/_/, $a);
	@Y = split(/_/, $b);

	if($X[0] ne $Y[0]){return($X[0] cmp $Y[0])}
	else{return($X[1] <=> $Y[1])}
	}
#sub to sort final print list
########################################


#################################################################
#sub to record the original array
sub RecordArray{
	@ArrayPrintOut = ();
	until($CrackLine == $#CrackData){
		$CrackLine++;
		$AddingArrayPrintOut = $CrackData[$CrackLine];
		chomp($AddingArrayPrintOut);
		push(@ArrayPrintOut, $AddingArrayPrintOut);
		}
	}
#end of sub to record the original array
#################################################################


######################################################
#sub to get the run-time
sub EndTime {
	@EndTime = localtime(time);

	if($EndTime[0] -  $StartTime[0] >= 0){
		$RunSecs = $EndTime[0] -  $StartTime[0]
		}
	else{
		$RunSecs = ($EndTime[0] -  $StartTime[0]) + 60;
		$EndTime[1]--;
		}

	if($EndTime[1] -  $StartTime[1] >= 0){
		$RunMins = $EndTime[1] -  $StartTime[1]
		}
		else{
		$RunMins = ($EndTime[1] -  $StartTime[1]) + 60;
		$EndTime[2]--;
		}

	if($EndTime[2] -  $StartTime[2] >= 0){
		$RunHours = $EndTime[2] -  $StartTime[2]
		}
	else{
		$RunHours = ($EndTime[2] -  $StartTime[2]) + 24;
		$EndTime[7]--;
		}

	if($EndTime[7] -  $StartTime[7] >= 0){
		$RunDays = $EndTime[7] -  $StartTime[7]
		}
	else{
		$RunDays = ($EndTime[7] -  $StartTime[7]) + 365
		}

	print "\nRun took $RunDays" . "d, $RunHours" . "h, $RunMins" . "m, $RunSecs" . "s\n\n";
	print OUTPUT "\n\n======================================";
	print OUTPUT "\n\nRun took $RunDays" . "d, $RunHours" . "h, $RunMins" . "m, $RunSecs" . "s\n\n";
	}
#end of sub to get the run-time
######################################################


}
########################################################
########################################################
#End of sub containing the whole program!




#below are two global subs to open an outfile and get a create-date



###############################################
#Sub to create various output files
sub OpenOut {
	while(1){
		print "\nSpecify or create $FileTypeToOpen file name, or Quit (q)\n";
		print "Filenames of more than 25 characters will be shortened\n   ";

		$OutFile = <stdin>;
		chomp($OutFile);
		$OutFile =~ tr/A-Z/a-z/;
		@PathOutFile = fileparse($OutFile, @SuffixList);


		$OutFile = $PathOutFile[0];

		$lengthtest=0;
		if($OutFile eq "q"){die "\nProgram quit"};

		&Out_length_check;
		unless(length($OutFile) == 0){last}
		}

	$OutFile = $OutFile . $Extension;

	if ($lengthtest == 1){
		print "\n$OldOutFile changed...\nOutput file saved as $OutFile\n\n"
		}

	$OutType = ">>";

	if($PathOutFile[1] eq $LocalPath){
		$PathOutFile[1] = $PathOriginalInFile[1];
			}
	$OutFile = $PathOutFile[1] . $OutFile;

	while(1){
		if ((-e $OutFile) != 0){
			print "\nThe file $OutFile already exists.  Append (a) or Replace (r)\n   ";
			$Exists = <stdin>;
			chomp($Exists);
			$Exists =~ tr/A-Z/a-z/;
			if($Exists eq "a"){
				$OutType = ">>";
				last
				}
			elsif($Exists eq "r"){
				$OutType = ">";
				last
				}
			else {next}
			}
		last}
	}

#end of sub to open output file
#######################################################

#######################################################
#sub to check length of output files
sub Out_length_check {
	# make sure output file is of the right length
	$OldOutFile = $OutFile;

	if(length($OutFile) > 25){
		until(length($OutFile) <= 25){chop $OutFile}
		$lengthtest=1;
		}
	}
#end of sub to check length of output files
#######################################################






######################################################
#sub to get the create-date
sub CreateDate {
	($CreateSec, $CreateMin, $CreateHour, $CreateDate, $CreateMonth, $CreateYear, $CreateDay, $Used, $Used) = localtime(time);
	@Days = ('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');
	@Months = ('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
	$CreateYear = $CreateYear + 1900;
	if(($CreateDate =~ /1$/)and($CreateDate !~ /11/)){$CreateDate = $CreateDate . "st"}
	elsif(($CreateDate =~ /2$/)and($CreateDate !~ /12/)){$CreateDate = $CreateDate . "nd"}
	elsif(($CreateDate =~ /3$/)and($CreateDate !~ /13/)){$CreateDate = $CreateDate . "rd"}
	else{$CreateDate = $CreateDate . "th"}
	}
#end of sub to get the create-date
######################################################
